' This is the action list displayed by the ScribbleControl's smart tag.
Imports System.ComponentModel.Design
Imports System.Drawing.Design
Imports System.Math

Public Class ScribbleActionList
    Inherits DesignerActionList

    ' The control we are manipulating.
    Private m_Scribble As ScribbleControl

    Private m_DesignerService As DesignerActionUIService = Nothing

    ' Save a reference to our control.
    Public Sub New(ByVal component As IComponent)
        MyBase.New(component)

        m_Scribble = DirectCast(component, ScribbleControl)

        ' Save a reference to DesignerActionUIService we can refresh it.
        m_DesignerService = _
            CType(GetService(GetType(DesignerActionUIService)), _
                DesignerActionUIService)
    End Sub

#Region "Smart Tag Items"

    ' LineColor.
    <EditorAttribute(GetType(ColorIndexEditor), GetType(UITypeEditor))> _
    Public Property LineColor() As ScribbleControl.LineColors
        Get
            Return m_Scribble.LineColor
        End Get
        Set(ByVal value As ScribbleControl.LineColors)
            GetType(ScribbleControl).GetProperty("LineColor").SetValue(m_Scribble, value, Nothing)
            m_Scribble.Invalidate()
        End Set
    End Property

    ' BackColor.
    Public Property BackColor() As Color
        Get
            Return m_Scribble.BackColor
        End Get
        Set(ByVal value As Color)
            GetType(ScribbleControl).GetProperty("BackColor").SetValue(m_Scribble, value, Nothing)
            m_Scribble.Invalidate()
        End Set
    End Property

    ' LineWidth.
    <EditorAttribute(GetType(LineWidthEditor), GetType(UITypeEditor))> _
    Public Property LineWidth() As Integer
        Get
            Return m_Scribble.LineWidth
        End Get
        Set(ByVal value As Integer)
            GetType(ScribbleControl).GetProperty("LineWidth").SetValue(m_Scribble, value, Nothing)
            m_Scribble.Invalidate()
        End Set
    End Property

    ' Make a star Polyline.
    Public Sub DrawStar()
        m_Scribble.PolyPolyline.Clear()
        m_Scribble.PolyPolyline.AddPolyline()

        Dim cx As Integer = m_Scribble.ClientSize.Width \ 2
        Dim cy As Integer = m_Scribble.ClientSize.Height \ 2
        Dim dtheta As Double = 2 * PI * 2 / 5
        For i As Integer = 0 To 6
            Dim x As Integer = CInt(cx + Cos(i * dtheta - PI / 2) * cx)
            Dim y As Integer = CInt(cy + Sin(i * dtheta - PI / 2) * cy)
            m_Scribble.PolyPolyline.AddPoint(x, y)
        Next i

        m_Scribble.Invalidate()

        ' Refresh the smart tag list to show the new Polyline count.
        Me.m_DesignerService.Refresh(Me.Component)
    End Sub

    ' Make a spiral Polyline.
    Public Sub DrawSpiral()
        m_Scribble.PolyPolyline.Clear()
        m_Scribble.PolyPolyline.AddPolyline()

        Dim cx As Integer = m_Scribble.ClientSize.Width \ 2
        Dim cy As Integer = m_Scribble.ClientSize.Height \ 2
        Dim theta As Double = 0
        Const NUM_POINTS As Integer = 100
        Dim dtheta As Double = 4 * PI / NUM_POINTS
        Dim rx As Double = 0
        Dim drx As Double = cx / NUM_POINTS
        Dim ry As Double = 0
        Dim dry As Double = cy / NUM_POINTS
        For i As Integer = 0 To NUM_POINTS - 1
            Dim x As Integer = CInt(cx + Cos(theta) * rx)
            Dim y As Integer = CInt(cy + Sin(theta) * ry)
            m_Scribble.PolyPolyline.AddPoint(x, y)
            theta += dtheta
            rx += drx
            ry += dry
        Next i

        m_Scribble.Invalidate()

        ' Refresh the smart tag list to show the new Polyline count.
        Me.m_DesignerService.Refresh(Me.Component)
    End Sub

#End Region ' Smart Tag Items

    ' Return a collection representing the smart tag items.
    Public Overrides Function GetSortedActionItems() As DesignerActionItemCollection
        Dim items As New DesignerActionItemCollection()

        ' Define section headers.
        items.Add(New DesignerActionHeaderItem("Appearance"))
        items.Add(New DesignerActionHeaderItem("Information"))

        ' Appearance entries.
        items.Add( _
            New DesignerActionPropertyItem( _
                "BackColor", _
                "Back Color", _
                "Appearance", _
                "Sets the background color."))
        items.Add( _
            New DesignerActionPropertyItem( _
                "LineColor", _
                "Line Color", _
                "Appearance", _
                "Sets the line drawing color."))
        items.Add( _
            New DesignerActionPropertyItem( _
                "LineWidth", _
                "Line Width", _
                "Appearance", _
                "Sets the line thickness."))

        ' The DrawStar subroutine.
        items.Add( _
            New DesignerActionMethodItem( _
                Me, _
                "DrawStar", _
                "Draw Star", _
                "Appearance", _
                "Draws a star.", _
                False))

        ' The DrawSpiral subroutine.
        ' Also appears in the context menu.
        items.Add( _
            New DesignerActionMethodItem( _
                Me, _
                "DrawSpiral", _
                "Draw Spiral", _
                "Appearance", _
                "Draws a spiral.", _
                True))

        ' Information entries.
        Dim txt As String = "# Polylines: " & m_Scribble.PolyPolyline.Polylines.Length
        items.Add( _
            New DesignerActionTextItem( _
                txt, _
                "Information"))

        Return items
    End Function

End Class
